package eu.lindenbaum.lab.racetest;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import co.paralleluniverse.actors.BasicActor;
import co.paralleluniverse.actors.ExitMessage;
import co.paralleluniverse.actors.LifecycleMessage;
import co.paralleluniverse.fibers.SuspendExecution;

class ParentActor extends BasicActor<Message, Void> {
  private final Map<Object, ChildActor> watched = new HashMap<>();

  final String id;

  ParentActor(String id) {
    this.id = id;
  }

  @Override
  protected Void doRun() throws InterruptedException, SuspendExecution {
    int counter = 0;

    for(;;) {
      Message message = receive();

      if (message instanceof LocalExitMessage) {
        System.err.println("[Parent " + id +  "] Exiting.");
        return null;
      }

      if (message instanceof CreateChildMessage) {
        ChildActor childActor = new ChildActor(id,  Integer.toString(++counter));
        childActor.spawn();
        watched.put(watch(childActor.ref()), childActor);
        System.err.println("[Parent " + id +  "] Created, started and watched " + childActor.id);
      }

      if (message instanceof ShutdownMessage) {
        ArrayList<ChildActor> actors = new ArrayList<>(watched.values());
        for (ChildActor childActor : actors) {
          System.err.println("[Parent " + id +  "] Asking shutdown of " + childActor.id);
          childActor.ref().send(ShutdownMessage.INSTANCE);
        }
      }
    }
  }

  @Override
  protected Message handleLifecycleMessage(LifecycleMessage m) {
    if (m instanceof ExitMessage) {
      watched.remove(((ExitMessage) m).getWatch());
      System.err.println("[Parent " + id +  "] Child exited. Children remaining: " + watched.size());
      if (watched.size() == 0) {
        System.err.println("[Parent " + id +  "] All children exited.");
        return new LocalExitMessage();
      }
    }
    return super.handleLifecycleMessage(m);
  }


  private static class LocalExitMessage implements Message {}
}
