package eu.lindenbaum.lab.racetest;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import co.paralleluniverse.actors.BasicActor;
import co.paralleluniverse.actors.ExitMessage;
import co.paralleluniverse.actors.LifecycleMessage;
import co.paralleluniverse.fibers.SuspendExecution;

class RootActor extends BasicActor<Message, Void> {
  private final Map<Object, ParentActor> watched = new HashMap<>();
  private final Map<String, ParentActor> parents = new HashMap<>();

  @Override
  protected Void doRun() throws InterruptedException, SuspendExecution {
    int counter = 0;

    while (true) {
      Message message = receive();

      if (message instanceof LocalExitMessage) {
        System.err.println("[ROOT] Exiting.");
        return null;
      }

      if (message instanceof CreateParentMessage) {
        ParentActor parentActor = new ParentActor(Integer.toString(++counter));
        parentActor.spawn();
        parents.put(parentActor.id, parentActor);
        watched.put(watch(parentActor.ref()), parentActor);
        System.err.println("[ROOT] Created, started and watched parent " + parentActor.id);
      } else if (message instanceof CreateChildForParentMessage) {
        CreateChildForParentMessage m = (CreateChildForParentMessage) message;
        System.err.println("[ROOT] Asking creation of a new child for parent " + m.parentId);
        parents.get(m.parentId).ref().send(CreateChildMessage.INSTANCE);
      } else if (message instanceof ShutdownMessage) {
        ArrayList<ParentActor> actors = new ArrayList<>(watched.values());
        for (ParentActor parentActor : actors) {
          System.err.println("[ROOT] Asking shutdown of " + parentActor.id);
          parentActor.ref().send(ShutdownMessage.INSTANCE);
        }
      }
    }
  }

  @Override
  protected Message handleLifecycleMessage(LifecycleMessage m) {
    if (m instanceof ExitMessage) {
      watched.remove(((ExitMessage) m).getWatch());
      System.err.println("[ROOT] Parent exited. Parents remaining: " + watched.size());
      if (watched.size() == 0) {
        System.err.println("[ROOT] All parents exited.");
        return new LocalExitMessage();
      }
    }
    return super.handleLifecycleMessage(m);
  }


  private static class LocalExitMessage implements Message {}
}
